<?php

/**
 * @file
 * Contains \Drupal\Core\Access\CsrfAccessCheck.
 */

namespace Drupal\Core\Access;

use Drupal\Core\Routing\Access\AccessInterface as RoutingAccessInterface;
use Symfony\Component\Routing\Route;
use Symfony\Component\HttpFoundation\Request;

/**
 * Allows access to routes to be controlled by a '_csrf_token' parameter.
 *
 * To use this check, add a "token" GET parameter to URLs of which the value is
 * a token generated by \Drupal::csrfToken()->get() using the same value as the
 * "_csrf_token" parameter in the route.
 */
class CsrfAccessCheck implements RoutingAccessInterface {

  /**
   * The CSRF token generator.
   *
   * @var \Drupal\Core\Access\CsrfTokenGenerator
   */
  protected $csrfToken;

  /**
   * Constructs a CsrfAccessCheck object.
   *
   * @param \Drupal\Core\Access\CsrfTokenGenerator $csrf_token
   *   The CSRF token generator.
   */
  function __construct(CsrfTokenGenerator $csrf_token) {
    $this->csrfToken = $csrf_token;
  }

  /**
   * Checks access based on a CSRF token for the request.
   *
   * @param \Symfony\Component\Routing\Route $route
   *   The route to check against.
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The request object.
   *
   * @return string
   *   A \Drupal\Core\Access\AccessInterface constant value.
   */
  public function access(Route $route, Request $request) {
    // If this is the controller request, check CSRF access as normal.
    if ($request->attributes->get('_controller_request')) {
      // @todo Remove dependency on the internal _system_path attribute:
      //   https://www.drupal.org/node/2293501.
      return $this->csrfToken->validate($request->query->get('token'), $request->attributes->get('_system_path')) ? static::ALLOW : static::KILL;
    }

    // Otherwise, this could be another requested access check that we don't
    // want to check CSRF tokens on.
    $conjunction = $route->getOption('_access_mode') ?: AccessManagerInterface::ACCESS_MODE_ANY;
    // Return ALLOW if all access checks are needed.
    if ($conjunction == AccessManagerInterface::ACCESS_MODE_ALL) {
      return static::ALLOW;
    }
    // Return DENY otherwise, as another access checker should grant access
    // for the route.
    else {
      return static::DENY;
    }
  }

}
